<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 * This is the main engine.
 */

/**
 * Master controller.
 */
class PHPDS_controller extends PHPDS_dependant
{
	protected $_POST;
	protected $_GET;

	public function construct()
	{
		$this->_POST = empty($_POST) ? array() : $_POST;
		$this->_GET = empty($_GET) ? array() : $_GET;
	}

	/**
	 * Return a value from the _POST meta array
	 *
	 * @date 20101016 (v1.0) (greg) added
	 * @version 1.0
	 * @author greg
	 *
	 * @param string|null $key the name of the post variable to fetch; if null, the entire array is returned
	 * @param mixed|array $default a default value to return when the post variable is not set; when returning the entire array, an array can be given here with default values
	 * @param integer $options
	 *
	 * @return scalar|array the content of the post variable or the whole array, possibly with default value(s)
	 */
	public function POST($key = null, $default = null, $options = 0)
	{
		if (!empty($key)) {
			return (isset($this->_POST[$key])) ? $this->_POST[$key] : $default;
		} else {
			if (is_array($default)) return array_merge($default, $this->_POST);
			else return $this->_POST;
		}
	}

	/**
	 * Return a value from the _GET meta array
	 *
	 * @date 20101016 (v1.0) (greg) added
	 * @version 1.0
	 * @author greg
	 *
	 * @param string|null $key the name of the get variable to fetch; if null, the entire array is returned
	 * @param mixed|array $default a default value to return when the get variable is not set; when returning the entire array, an array can be given here with default values
	 * @param integer $options
	 *
	 * @return scalar|array the content of the get variable or the whole array, possibly with default value(s)
	 */
	public function GET($key = null, $default = null, $options = 0)
	{
		if (!empty($key)) {
			return (isset($this->_GET[$key])) ? $this->_GET[$key] : $default;
		} else {
			if (is_array($default)) return array_merge($default, $this->_GET);
			else return $this->_GET;
		}
	}

	/**
	 * Does security check and runs controller.
	 *
	 * @return mixed
	 */
	public function run()
	{
		(is_object($this->security)) ? $this->security->securityIni() : exit('Access Denied!');
		if (PU_isAJAX ()) {
			PU_exitToAJAX($this->viaAJAX());
		} else {
			$this->execute();
		}
	}

	/**
	 * This method is meant to be the entry point of your class. Most checks and cleanup should have been done by the time it's executed
	 *
	 * @return whatever
	 */
	public function execute()
	{
		// your code here
	}

	public function viaAJAX()
	{
		// your code here
	}
}
