<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */
/**
 * This class extracts and organizes the main menus from the database and displayes it to the user. Shows only where permissions apply.
 * This is the complete menu extraction class.
 * @deprecated
 * @author Jason Schoeman
 */
class navigation extends PHPDS_dependant
{
	/**
	 * Determines what the menu item should be named.
	 *
	 * @param string $replacement_name
	 * @param string $menu_link
	 * @param int $menu_id
	 * @return string
	 */
	public function determine_menu_name ($replacement_name = false, $menu_link, $menu_id = false)
	{
		return $this->determineMenuName($replacement_name, $menu_link, $menu_id);
	}

	/**
	 * Compiles menu items in order.
	 *
	 * @return string
	 * @author Jason Schoeman
	 */
	public function write_menu ()
	{
		// Define.
		$menu = false;
		// Make sure array is not empty, otherwise error will be produced.
		if (! empty($this->navigation)) {
			// Get menu group to compare against.
			if (! empty($this->navigation[$this->configuration['m']]['is_parent'])) {
				$menu_group = $this->configuration['m'];
			} else {
				if (! empty($this->navigation[$this->configuration['m']]['parent_menu_id'])) {
					$menu_group = $this->navigation[$this->configuration['m']]['parent_menu_id'];
				} else {
					$menu_group = 0;
				}
			}
			// Start dynamic render engine.
			foreach ($this->navigation as $m) {
				// Check if active item is parent.
				if ($this->show_menu($m['hide'], $m['menu_id']) && $this->navigation[$m['menu_id']]['parent_menu_id'] == $menu_group) {
					// Swith menu types.
					if ($m['is_parent'] == 1) {
						// Root item with Children.
						$menu .= "<li class=\"dir\">{$m['url']}<ul>{$this->call_family($m['menu_id'])}</ul></li>";
					} else {
						// Check if current looped url is active.
						($m['menu_id'] == $this->configuration['m']) ? $url_active = 'active' : $url_active = 'inactive';
						// Root item with no Children.
						$menu .= "<li id=\"$url_active\">{$m['url']}</li>";
					}
				}
			}
			// If no menu item is present, at least call its parent!
			if (empty($menu)) {
				$menu = "<li id=\"active\">{$this->navigation[$this->configuration['m']]['url']}</li>";
			}
		}
		// Return Menu.
		return $menu;
	}

	/**
	 * Assists write_menu in calling menu children.
	 *
	 * @param int $menu_id
	 */
	public function call_family ($menu_id = false)
	{
		// Define.
		$menu = false;
		// Only loop when we have something.
		if (! empty($this->child[$menu_id])) {
			// Loop menu items finding its children.
			foreach ($this->child[$menu_id] as $m) {
				// Should item be shown.
				if ($this->show_menu($this->navigation[$m]['hide'], $m)) {
					// Check if we have a parent, if we do, we need to call her children.
					if ($this->navigation[$m]['is_parent'] == 1) {
						// Child item with Children.
						$menu .= "<li class=\"dir\">{$this->navigation[$m]['url']}<ul>{$this->call_family($m)}</ul></li>";
					} else {
						// Check if current looped url is active.
						($m == $this->configuration['m']) ? $url_active = 'active' : $url_active = 'inactive';
						// Child item without Children.
						$menu .= "<li id=\"$url_active\">{$this->navigation[$m]['url']}</li>";
					}
				}
			}
		}
		return $menu;
	}

	/**
	 * This method compiles the history tree seen, this is the tree that the user sees expand when going deeper into menu levels.
	 * On the default template this is the navigation link string top left above the menus.
	 *
	 * @return string
	 */
	public function generate_history_tree ()
	{
		// Check if user has permission to view Jump Menu!
		if (isset($this->navigation['940041356']['menu_id'])) {
			$jump_menu = '<a href="' . $this->load_page(940041356) . '" id="jump_history_menu">' . $this->lang['940041356'] . '</a>';
		} else {
			$jump_menu = false;
		}
		// Front Page.
		$main_item = '<a href="' . $this->configuration['absolute_url'] . '" id="folder_root">' . ___('Front Page') . '</a>' . $jump_menu;
		// Call breadcrumb method.
		$this->callback_parent_item($this->configuration['m']);
		// Set.
		$history_url = false;
		foreach (array_reverse($this->breadcrumbArray, true) as $key => $breadcrumb_id) {
			if (! empty($this->navigation[$breadcrumb_id]['menu_name']) && (($breadcrumb_id != $this->configuration['m']) || ($key != 0))) {
				$history_url .= '<a href="' . $this->load_page($breadcrumb_id) . '" id="folder_history_tree">' . $this->navigation[$breadcrumb_id]['menu_name'] . '</a>';
			}
		}
		// Go up one level.
		if (! empty($this->navigation[$this->configuration['m']]['parent_menu_id'])) {
			$up_parent_id = $this->navigation[$this->configuration['m']]['parent_menu_id'];
		} else {
			$up_parent_id = 0;
		}
		// Create Up link.
		if (! empty($up_parent_id)) {
			// Create level up icon.
			$up = '<a href="' . $this->load_page($up_parent_id) . '" id="up_history_tree">' . ___('Up') . '</a>';
		} else {
			$up = false;
		}
		// Do we have the history URL?
		if (empty($history_url)) $history_url = false;
		// Return items, in order.
		return $main_item . $history_url . $up;
	}

	/**
	 * Method assists method generate_history_tree in getting breadcrumb links.
	 *
	 * @param integer
	 * @return array
	 */
	private function callback_parent_item ($menu_id_)
	{
		// Check what called menu items parent id is.
		if (! empty($this->navigation[$menu_id_]['parent_menu_id'])) {
			$recall_parent_menu_id = $this->navigation[$menu_id_]['parent_menu_id'];
		} else {
			$recall_parent_menu_id = 0;
		}
		// Return results.
		$this->breadcrumbArray[] = $menu_id_;
		// Call function itself to see if this item has a parent item.
		if ($recall_parent_menu_id != 0) {
			$this->callback_parent_item($recall_parent_menu_id);
		}
	}

	/**
	 * Returns true if menu should show.
	 *
	 * @deprecated
	 * @param integer $hide_type
	 * @param integer $menu_id
	 */
	public function show_menu ($hide_type, $menu_id = false)
	{
		return $this->showMenu($hide_type, $menu_id);
	}

	/**
	 * Convert plugin file location to unsigned CRC32 value. This is unique and allows one to locate a menu item from location as well.
	 *
	 * @deprecated use createMenuId
	 * @param string The plugin folder the file is in.
	 * @return integer
	 * @author Jason Schoeman
	 */
	public function create_menu_id ($path)
	{
		return $this->createMenuId($path);
	}

	/**
	 * Returns the correct string for use in href when creating a link for a menu id. Will return sef url if possible.
	 * Will return self url when no menu id is given. No starting & or ? is needed, this gets auto determined!
	 * If left empty it will return current active menu.
	 *
	 * @deprecated
	 * @param integer The menu id to load.
	 * @param string extend_url
	 * @param boolean strip_trail Will strip unwanted empty operators at the end.
	 * @return string
	 * @author Jason Schoeman
	 */
	public function load_page ($menu_id = false, $extend_url = '', $strip_trail = '')
	{
		// Should we assign menu id.
		if (empty($menu_id)) $menu_id = $this->configuration['m'];
		// Can we use sef url?
		if (! empty($this->configuration['sef_url']) && ! empty($this->navigation["$menu_id"]['alias'])) {
			// Add extra string. Thought not 100% correct, this is for backwards compatibility.
			if (! empty($extend_url)) {
				$extend_url = "?$extend_url";
			} else if (! empty($strip_trail)) {
				$extend_url = false;
			} else {
				$extend_url = '?';
			}
			$url_append = empty($this->configuration['url_append']) ? '' : $this->configuration['url_append'];
			// Menu id is set for sef mode, so return correct integer id.
			$url = $this->navigation["$menu_id"]['alias'] . $url_append . "$extend_url";
		} else {
			// Add extra string.
			if (! empty($extend_url)) {
				$extend_url = "&$extend_url";
			} else {
				$extend_url = false;
			}
			// Menu id is an integer.
			$url = 'index.php?m=' . "$menu_id" . "$extend_url";
		}
		if (! empty($url)) {
			return $this->configuration['absolute_url'] . "/$url";
		} else {
			return false;
		}
	}

	/**
	 * This method saves the current URL with the option to add more $this->security->get variables like ("&variable1=1&variable2=2")
	 * This is mostly used for when additional $this->security->get variables are required! Usefull when using forms.
	 *
	 * @param string Add more $this->security->get variables like ("&variable1=1&variable2=2")
	 * @return string
	 * @deprecated This method is not needed anymore.
	 * @author Jason Schoeman
	 */
	public function self_url ($extra_get_variables = false)
	{
		// Workaround.
		return $this->load_page(false, $extra_get_variables, true);
	}

	/**
	 * Creates a link that looks like all the other submit buttons.
	 *
	 * @deprecated
	 * @param string $url
	 * @return string
	 */
	public function generic_button ($name, $url = false)
	{
		return '<input type="button" value="' . $name . '" name="generic" onClick="window.location=\'' . $url . '\' " class="button">';
	}
}
?>