<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Controller Class: User Group Administration
 * @author Jason Schoeman
 * @return string
 */
class UserGroupAdmin extends PHPDS_controller
{

	/**
	 * Execute Controller
	 * @author Jason Schoeman
	 */
	public function execute()
	{
		// Define.
		$user_group_id = 0;
		$user_group_name = '';
		$user_group_note = '';
		$alias = '';
		$parent_group_id = 0;

		// Lets create header information.
		if (!empty($this->security->get['eg']) || !empty($this->security->post['save'])) {
			$this->template->heading(_('Edit Group Names'));
			$this->template->info(_('Used to edit existing user groups. User Groups allows the system to restrict user access to only access data belonging to the same group. Allowing the creation of data accessing policies. These settings does not restrict script accessing. If you assign a user to a parent group, user will be able to access all child groups as well.'));

			// Set to edit mode.
			$edit_mode = true;
			// Set edit group id.
			(!empty($this->security->get['eg'])) ? $eg = $this->security->get['eg'] : $eg = 0;
		} else {
			$this->template->heading(_('Add Groups'));
			$this->template->info(_('Used to add user groups. User Groups allows the system to restrict user access to only access data belonging to the same group. Allowing the creation of data accessing policies. These settings does not restrict script accessing. If you assign a user to a parent group, user will be able to access all child groups as well.'));

			// Set to edit mode.
			$edit_mode = false;
			// Set edit group id.
			$eg = 0;
		}
		// If item gets saved.
		if (!empty($this->security->post['save']) || !empty($this->security->post['new'])) {
			if (!empty($this->security->post['user_group_id'])) {
				$user_group_id = $this->security->post['user_group_id'];
			} else {
				$user_group_id = 0;
			}
			$parent_group_id = $this->security->post['parent_group_id'];
			$user_group_name = $this->security->post['user_group_name'];
			$user_group_note = $this->security->post['user_group_note'];
			$alias = $this->security->post['alias'];
			$translated_group_name = $user_group_name;

			// Some minor error checking.
			if (empty($user_group_name)) {
				$this->template->warning(_('You need to provide values for all fields, please try again.'));
				$error[1] = true;
			} else {
				// Does the record already exist.
				if ($this->db->doesRecordExist('_db_core_user_groups', 'user_group_name', "$user_group_name", 'user_group_id', "$user_group_id") == true) {
					$this->template->warning(sprintf(_('There is already a role record named %s. Cannot duplicate.'), $user_group_name));
					$error[2] = true;
				}
			}
			// Check if user can save this group.
			if (!empty($user_group_id) && !$this->user->belongsToGroup(false, $user_group_id)) {
				$this->template->warning('You cannot save this group, it does not belong to you!');
				$error[3] = true;
			}
			// If everything is ok we can now save.
			if (empty($error)) {

				$user_group_id = $this->db->invokeQuery('PHPDS_writeGroupQuery', $user_group_id, $user_group_name, $user_group_note, $parent_group_id, $alias);

				// Set variable global for hooks.
				$this->template->global['user_group_id'] = $user_group_id;
				// Oops but before we can save we first need to check if this user was not inserted already.
				if (empty($this->security->post['user_group_id'])) {
					// Also the user who submitted this group, we want to add him to this group.
					$this->db->invokeQuery('PHPDS_writeExtraGroupQuery', $this->configuration['user_id'], $user_group_id);
					// Show that we added this user to this group.
					$this->template->ok(sprintf(_('User id %s was added to group id %s.'), $this->configuration['user_id'], $user_group_id));
				}

				// Show item updated.
				$this->template->ok(sprintf(_('You have saved user group %s.'), $translated_group_name));
			}
			// Save and new.
			if (!empty($this->security->post['new'])) {
				$user_group_id = 0;
				$user_group_name = '';
				$user_group_note = '';
				$alias = '';
			}
		}
		// Edit the group and load data for group.
		if (!empty($edit_mode) && !empty($eg)) {
			// Check if user can save this group.
			if ($this->user->belongsToGroup(false, $eg)) {
				// If item is in edit mode.
				$select_user_group_array = $this->db->invokeQuery('PHPDS_readGroupQuery', $eg);

				$user_group_id = $select_user_group_array['user_group_id'];
				$parent_group_id = $select_user_group_array['parent_group_id'];
				$user_group_name = $select_user_group_array['user_group_name'];
				$user_group_note = $select_user_group_array['user_group_note'];
				$alias = $select_user_group_array['alias'];
			} else {
				$this->template->warning(_('You cannot edit this group, it does not belong to you!'));
			}
		}

		// Create parent group options.
		$parent_group_option = $this->db->invokeQuery('PHPDS_readParentGroupQuery', $parent_group_id, $user_group_id);

		// Load views.
		$view = $this->factory('views');

		// Set Values.
		$view->set('self_url', $this->navigation->selfUrl());
		$view->set('user_group_id', $user_group_id);
		$view->set('parent_group_option', $parent_group_option);
		$view->set('user_group_name', $user_group_name);
		$view->set('user_group_note', $user_group_note);
		$view->set('alias', $alias);
		$view->set('post_validation', $this->security->postValidation());

		// Output Template.
		$view->show();
	}
}

return 'UserGroupAdmin';